(* Support file for generating events logs *)
(* Wil van der Aalst *)
(* www.vdaalst.com *)
(* feel free to reuse with acknowledgements *)

(* TIME unit is seconds*)
val minute = 60.0;
val hour = 60.0*minute;
val day = 24.0*hour;
val week = 7.0*day;

fun Mtime() = ModelTime.time():time;

fun monday_jan_5_2015() = 16440.0*day - 1.0*hour;

fun start_time() = monday_jan_5_2015();

fun print_start_time() = Date.fmt "%Y-%m-%d %H:%M:%S" (Date.fromTimeLocal(Time.fromReal(start_time())));

(* TIME OUTPUT mySQL*)
fun t2s(t) = Date.fmt "%Y-%m-%d %H:%M:%S" (Date.fromTimeLocal(Time.fromReal(t+start_time())));

(* TIME OUTPUT KEYVALUE*)
fun t2s_alt(t) = Date.fmt "%d-%m-%Y %H:%M:%S" (Date.fromTimeLocal(Time.fromReal(t+start_time())));

(* BETA DISTRIBUTION *)
fun ran_beta(low:real,high:real,a:real,b:real) = low + ((high-low)*beta(a,b)):real; 

fun mean_beta(low:real,high:real,a:real,b:real) = low + ((high-low)* (a/(a+b)));

fun mode_beta(low:real,high:real,a:real,b:real) = low + ((high-low)*((a-1.0)/(a+b-2.0)));

fun var_beta(low:real,high:real,a:real,b:real) = ((high-low)*(high-low)* ((a*b)/((a+b)*(a+b)*(a+b+1.0))));

fun stdev_beta(low:real,high:real,a:real,b:real) = Math.sqrt(var_beta(low,high,a,b));

fun timetik() = exponential(1.0/60.0);

(* TIME FUNCTIONS *)

fun t2date(t) = Date.fromTimeLocal(Time.fromReal(t+start_time()));


fun t2year(t) = Date.year(t2date(t)):int;
fun t2month(t) = Date.month(t2date(t)):Date.month;
fun t2day(t) = Date.day(t2date(t)):int;
fun t2hour(t) = Date.hour(t2date(t)):int;
fun t2minute(t) = Date.minute(t2date(t)):int;
fun t2second(t) = Date.second(t2date(t)):int;
fun t2weekday(t) = Date.weekDay(t2date(t)):Date.weekday;

fun t2monthstr(t) = Date.fmt "%b" (Date.fromTimeLocal(Time.fromReal(t+start_time())));
fun t2weekdaystr(t) = Date.fmt "%a" (Date.fromTimeLocal(Time.fromReal(t+start_time())));

fun remaining_time_hour(t) = hour - ((fromInt(t2minute(t))*minute) + fromInt(t2second(t)));



(* FILE HANDLING *)

val FILE = "./event_log.csv";
val SEP = ";";

fun list2string([]) = ""|
list2string(x::l) = x ^ (if l=[] then "" else SEP) ^ list2string(l);

fun create_log_file(l) = 
let
   val file_id = TextIO.openOut(FILE)
   val _ = TextIO.output(file_id, list2string(l)) 
   val _ = TextIO.output(file_id, if l = [] then "" else "\n")
in
   TextIO.closeOut(file_id)
end;


fun write_record(l) = 
let
   val file_id = TextIO.openAppend(FILE)
   val _ = TextIO.output(file_id, list2string(l))
   val _ = TextIO.output(file_id, "\n")
   
in
   TextIO.closeOut(file_id)
end;

(* EXAMPLE FORMAT *)

(* val header = ["case","activity","time"];

fun event(c,a) = write_record([Int.toString(c),a,t2s(Mtime())]);

fun event_alt(c,a) = write_record([Int.toString(c),a,t2s_alt(Mtime())]);*)

val header = ["case","activity","time"];

fun event(c,a) = write_record([Int.toString(c),a,t2s(Mtime())]);

(* Support file for generating events logs *)
(* Wil van der Aalst *)
(* www.vdaalst.com *)
(* feel free to reuse with acknowledgements *)







